@@ -1,4 +1,5 @@
 from bnlearn.bnlearn import (
+    to_bayesianmodel,
     make_DAG,
     print_CPD,
     import_DAG,
@@ -10,18 +11,24 @@ from bnlearn.bnlearn import (
     adjmat2vec,
     vec2adjmat,
     df2onehot,
-    to_BayesianModel,
+    topological_sort,
+    predict,
+    query2df,
+    _dag2adjmat,
+    _filter_df,
 )
 
 # Import function in new level
 import bnlearn.structure_learning as structure_learning
 import bnlearn.parameter_learning as parameter_learning
 import bnlearn.inference as inference
+import bnlearn.network as network
+import bnlearn.confmatrix as confmatrix
 from packaging import version
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '0.3.16'
+__version__ = '0.3.21'
 
 try:
     import pgmpy
@@ -35,11 +42,11 @@ if version.parse(pgmpy.__version__) < version.parse("0.1.13"):
 # Version check
 import matplotlib
 if not version.parse(matplotlib.__version__) >= version.parse("3.3.4"):
-    print('[bnlearn] >Error: Matplotlib version should be >= v3.3.4.\nTry to: pip install -U matplotlib')
+    raise Exception('[bnlearn] >Error: Matplotlib version should be >= v3.3.4.\nTry to: pip install -U matplotlib')
 
 import networkx as nx
-if not version.parse(nx.__version__) >= version.parse("2.5"):
-    print('[bnlearn] >Error: networkx version should be >= 2.5.\nTry to: pip install -U networkx')
+if not version.parse(nx.__version__) > version.parse("2.5"):
+    raise Exception('[bnlearn] >Error: networkx version should be > 2.5.\nTry to: pip install -U networkx')
 
 
 # module level doc-string
