@@ -9,26 +9,30 @@
 
 # %% Libraries
 import os
+import wget
+import zipfile
+import json
+import itertools
 import pandas as pd
 import numpy as np
 import networkx as nx
 import matplotlib.pyplot as plt
-import json
 from ismember import ismember
+from tqdm import tqdm
 
 from pgmpy.models import BayesianModel
 from pgmpy.factors.discrete import TabularCPD
 from pgmpy.sampling import BayesianModelSampling  # GibbsSampling
 
 from pgmpy import readwrite
-import bnlearn.helpers.network as network
-
-curpath = os.path.dirname(os.path.abspath(__file__))
-PATH_TO_DATA = os.path.join(curpath, 'data')
+# import bnlearn.network as network
+# from bnlearn.network import compare_networks, graphlayout, adjmat2graph
+# import bnlearn.inference as inference
+import bnlearn
 
 
-# %% Convert adjmat to bayesian model
-def to_BayesianModel(model, verbose=3):
+#%%  Convert adjmat to bayesian model
+def to_bayesianmodel(model, verbose=3):
     """Convert adjacency matrix to BayesianModel.
 
     Description
@@ -57,7 +61,7 @@ def to_BayesianModel(model, verbose=3):
         adjmat = model.get('adjmat', None)
     else:
         adjmat = model
-    if adjmat is None: raise Exception('[bnlearn] >Error: input for "to_BayesianModel" should be adjmat or a dict containing a key "adjmat".')
+    if adjmat is None: raise Exception('[bnlearn] >Error: input for "to_bayesianmodel" should be adjmat or a dict containing a key "adjmat".')
 
     if verbose>=3: print('[bnlearn] >Conversion of adjmat to BayesianModel.')
 
@@ -154,7 +158,7 @@ def print_CPD(DAG, checkmodel=False):
         elif 'BayesianModel' in str(type(DAG)):
             # print CPDs using Bayesian Parameter Estimation
             if len(DAG.get_cpds())==0:
-                print('[bnlearn] >This seems like an Bayesian DAG containing only edges, and no CPDs. Tip: use bnlearn.plot(DAG) to make a plot of the edges.')
+                raise Exception('[bnlearn] >Error! This is a Bayesian DAG containing only edges, and no CPDs. Tip: you need to specify or learn the CPDs. Try: DAG=bn.parameter_learning.fit(DAG, df). At this point you can make a plot with: bn.plot(DAG).')
                 return
             for cpd in DAG.get_cpds():
                 print("CPD of {variable}:".format(variable=cpd.variable))
@@ -172,8 +176,7 @@ def print_CPD(DAG, checkmodel=False):
 
 # %%
 def _check_model(DAG, verbose=3):
-    if verbose>=3:
-        print('[bnlearn] >Checking CPDs..')
+    if verbose>=3: print('[bnlearn] >Checking CPDs..')
     for cpd in DAG.get_cpds():
         # print(cpd)
         if not np.all(cpd.values.sum(axis=0)==1):
@@ -182,84 +185,6 @@ def _check_model(DAG, verbose=3):
         print('[bnlearn] >Check for DAG structure. Correct: %s' %(DAG.check_model()))
 
 
-# %% Make DAG
-def import_DAG(filepath='sprinkler', CPD=True, checkmodel=True, verbose=3):
-    """Import Directed Acyclic Graph.
-
-    Parameters
-    ----------
-    filepath : str, (default: sprinkler)
-        Pre-defined examples are depicted below, or provide the absolute file path to the .bif model file.. The default is 'sprinkler'.
-        'sprinkler', 'alarm', 'andes', 'asia', 'pathfinder', 'sachs', 'miserables', 'filepath/to/model.bif',
-    CPD : bool, optional
-        Directed Acyclic Graph (DAG). The default is True.
-    checkmodel : bool
-        Check the validity of the model. The default is True
-    verbose : int, optional
-        Print progress to screen. The default is 3.
-        0: None, 1: ERROR, 2: WARN, 3: INFO (default), 4: DEBUG, 5: TRACE
-
-    Returns
-    -------
-    dict containing model and adjmat.
-        model : BayesianModel
-        adjmat : Adjacency matrix
-
-    Examples
-    --------
-    >>> import bnlearn as bn
-    >>> model = bn.import_DAG('sprinkler')
-    >>> bn.plot(model)
-
-    """
-    out={}
-    model=None
-    filepath=filepath.lower()
-
-    # Load data
-    if filepath=='sprinkler':
-        model = _DAG_sprinkler(CPD=CPD)
-    elif filepath=='asia':
-        model = _bif2bayesian(os.path.join(PATH_TO_DATA, 'ASIA/asia.bif'), verbose=verbose)
-    elif filepath=='alarm':
-        model = _bif2bayesian(os.path.join(PATH_TO_DATA, 'ALARM/alarm.bif'), verbose=verbose)
-    elif filepath=='andes':
-        model = _bif2bayesian(os.path.join(PATH_TO_DATA, 'ANDES/andes.bif'), verbose=verbose)
-    elif filepath=='pathfinder':
-        model = _bif2bayesian(os.path.join(PATH_TO_DATA, 'PATHFINDER/pathfinder.bif'), verbose=verbose)
-    elif filepath=='sachs':
-        model = _bif2bayesian(os.path.join(PATH_TO_DATA, 'SACHS/sachs.bif'), verbose=verbose)
-    elif filepath=='water':
-        model = _bif2bayesian(os.path.join(PATH_TO_DATA, 'WATER/water.bif'), verbose=verbose)
-    elif filepath=='miserables':
-        f = open(os.path.join(PATH_TO_DATA, 'miserables.json'))
-        data = json.loads(f.read())
-        L=len(data['links'])
-        edges=[(data['links'][k]['source'], data['links'][k]['target']) for k in range(L)]
-        model=nx.Graph(edges, directed=False)
-    else:
-        if os.path.isfile(filepath):
-            model = _bif2bayesian(filepath, verbose=verbose)
-        else:
-            if verbose>=3: print('[bnlearn] >filepath does not exist! <%s>' %(filepath))
-            return(out)
-
-    # Setup adjacency matrix
-    adjmat = _dag2adjmat(model)
-
-    # Store
-    out['model']=model
-    out['adjmat']=adjmat
-
-    # check_model check for the model structure and the associated CPD and returns True if everything is correct otherwise throws an exception
-    if (model is not None) and CPD and checkmodel:
-        _check_model(out['model'], verbose=verbose)
-        if verbose>=4:
-            print_CPD(out)
-
-    return(out)
-
-
 # %% Convert DAG into adjacency matrix
 def _dag2adjmat(model, verbose=3):
     adjmat = None
@@ -403,14 +328,14 @@ def sampling(DAG, n=1000, verbose=3):
     if verbose>=3: print('[bnlearn] >Forward sampling for %.0d samples..' %(n))
 
     if len(DAG['model'].get_cpds())==0:
-        print('[bnlearn] >This seems like a DAG containing only edges, and no CPDs. Tip: use bn.parameter_learning.fit(DAG, df) to learn the CPDs first.')
+        raise Exception('[bnlearn] >Error! This is a Bayesian DAG containing only edges, and no CPDs. Tip: you need to specify or learn the CPDs. Try: DAG=bn.parameter_learning.fit(DAG, df). At this point you can make a plot with: bn.plot(DAG).')
         return
 
     # http://pgmpy.org/sampling.html
     infer_model = BayesianModelSampling(DAG['model'])
-    # inference = GibbsSampling(model['model'])
+    # inference1 = GibbsSampling(model['model'])
     # Forward sampling and make dataframe
-    df=infer_model.forward_sample(size=n, return_type='dataframe')
+    df=infer_model.forward_sample(size=n)
     return(df)
 
 
@@ -554,7 +479,7 @@ def compare_networks(model_1, model_2, pos=None, showfig=True, figsize=(15, 8),
         adjmat_diff : Adjacency matrix depicting the differences between the two input models.
 
     """
-    [scores, adjmat_diff] = network.compare_networks(model_1['adjmat'], model_2['adjmat'], pos=pos, showfig=showfig, width=figsize[0], height=figsize[1], verbose=verbose)
+    scores, adjmat_diff = bnlearn.network.compare_networks(model_1['adjmat'], model_2['adjmat'], pos=pos, showfig=showfig, width=figsize[0], height=figsize[1], verbose=verbose)
     return(scores, adjmat_diff)
 
 
@@ -597,7 +522,7 @@ def plot(model, pos=None, scale=1, figsize=(15, 8), verbose=3):
     if 'BayesianModel' in str(type(model)) or 'pgmpy' in str(type(model)):
         if verbose>=3: print('[bnlearn] >Plot based on BayesianModel')
         # positions for all nodes
-        pos = network.graphlayout(model, pos=pos, scale=scale, layout=layout, verbose=verbose)
+        pos = bnlearn.network.graphlayout(model, pos=pos, scale=scale, layout=layout, verbose=verbose)
         # Add directed edge with weigth
         # edges=model.edges()
         edges=[*model.edges()]
@@ -606,12 +531,12 @@ def plot(model, pos=None, scale=1, figsize=(15, 8), verbose=3):
     elif 'networkx' in str(type(model)):
         if verbose>=3: print('[bnlearn] >Plot based on networkx model')
         G = model
-        pos = network.graphlayout(G, pos=pos, scale=scale, layout=layout, verbose=verbose)
+        pos = graphlayout(G, pos=pos, scale=scale, layout=layout, verbose=verbose)
     else:
         if verbose>=3: print('[bnlearn] >Plot based on adjacency matrix')
-        G = network.adjmat2graph(model)
+        G = adjmat2graph(model)
         # Get positions
-        pos = network.graphlayout(G, pos=pos, scale=scale, layout=layout, verbose=verbose)
+        pos = graphlayout(G, pos=pos, scale=scale, layout=layout, verbose=verbose)
 
     # Bootup figure
     plt.figure(figsize=figsize)
@@ -637,6 +562,89 @@ def plot(model, pos=None, scale=1, figsize=(15, 8), verbose=3):
     out['G']=G
     return(out)
 
+# %%
+def topological_sort(adjmat, start=None):
+    """Topological sort.
+    
+    Description
+    -----------
+    Get nodes list in the topological sort order.
+
+    Parameters
+    ----------
+    adjmat : pd.DataFrame or bnlearn object.
+        Adjacency matrix.
+    start : str, optional
+        Start position. The default is None and the whole network is examined.
+
+    Returns
+    -------
+    list
+        Topological sort order.
+
+    Example
+    -----------
+    import bnlearn as bn
+    DAG = bn.import_DAG('sprinkler', verbose=0)
+    bn.topological_sort(DAG, 'Rain')
+    bn.topological_sort(DAG)
+
+
+    References
+    ----------
+    https://stackoverflow.com/questions/47192626/deceptively-simple-implementation-of-topological-sorting-in-python
+
+    """
+    # Convert to adjmat
+    if isinstance(adjmat, dict):
+        adjmat = adjmat.get('adjmat', None)
+    elif np.all(np.isin(adjmat.columns, ['source','target','weight'])):
+        adjmat = vec2adjmat(adjmat['source'], adjmat['target'])
+
+    # Convert to graph
+    graph = adjmat2dict(adjmat)
+    # Do the topological sort
+    seen = set()
+    stack = []    # path variable is gone, stack and order are new
+    order = []    # order will be in reverse order at first
+    if start is None:
+        q = list(graph)
+    else:
+        q = [start]
+    while q:
+        v = q.pop()
+        if v not in seen:
+            seen.add(v) # no need to append to path any more
+            q.extend(graph[v])
+
+            while stack and v not in graph[stack[-1]]: # new stuff here!
+                order.append(stack.pop())
+            stack.append(v)
+
+    return stack + order[::-1]
+
+
+# %%
+def adjmat2dict(adjmat):
+    """Convert adjacency matrix to dict.
+
+    Parameters
+    ----------
+    adjmat : pd.DataFrame
+        Adjacency matrix.
+
+    Returns
+    -------
+    graph : dict
+        Graph.
+
+    """
+    graph={}
+    rows=adjmat.index.values
+    for r in rows:
+        graph.update({r: list(rows[adjmat.loc[r,:]])})
+    return graph
+
 
 # %% Example data
 def import_example(data='sprinkler', n=10000, verbose=3):
@@ -646,7 +654,7 @@ def import_example(data='sprinkler', n=10000, verbose=3):
     ----------
     data : str, (default: sprinkler)
         Pre-defined examples.
-        'titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'pathfinder', 'sachs', 'water'
+        'titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'water'
     n : int, optional
         Number of samples to generate. The default is 1000.
     verbose : int, (default: 3)
@@ -658,21 +666,33 @@ def import_example(data='sprinkler', n=10000, verbose=3):
     df : pd.DataFrame()
 
     """
-    import wget
 
-    url = 'https://erdogant.github.io/datasets/'
-    if data=='sprinkler':
-        url=url + 'sprinkler.zip'
-    elif data=='titanic':
-        url=url + 'titanic_train.zip'
+    # Change name for downloading
+    if data=='titanic': data = 'titanic_train'
+    
+    # Download example dataset from github source
+    PATH_TO_DATA = _download_example(data, verbose=verbose)
+
+    # Import dataset
+    if (data=='sprinkler') or (data=='titanic_train'):
+        if verbose>=3: print('[bnlearn] >Import dataset..')
+        df = pd.read_csv(PATH_TO_DATA)
     else:
         try:
+            getPath = _unzip(PATH_TO_DATA, verbose=verbose)
             DAG = import_DAG(data, verbose=2)
             df = sampling(DAG, n=n, verbose=2)
         except:
             print('[bnlearn] >Error: Loading data not possible!')
             df = None
-        return df
+
+    return df
+
+#%% Download data from github source
+def _download_example(data, verbose=3):
+    # Set url location
+    url = 'https://erdogant.github.io/datasets/'
+    url=url + data+'.zip'
 
     curpath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
     PATH_TO_DATA = os.path.join(curpath, wget.filename_from_url(url))
@@ -681,13 +701,104 @@ def import_example(data='sprinkler', n=10000, verbose=3):
 
     # Check file exists.
     if not os.path.isfile(PATH_TO_DATA):
-        if verbose>=3: print('[bnlearn] >Downloading example dataset..')
+        if verbose>=3: print('[bnlearn] >Downloading example [%s] dataset..' %(data))
         wget.download(url, curpath)
+    
+    return PATH_TO_DATA
 
-    # Import local dataset
-    if verbose>=3: print('[bnlearn] >Import dataset..')
-    df = pd.read_csv(PATH_TO_DATA)
-    return df
+# %% Make DAG
+def import_DAG(filepath='sprinkler', CPD=True, checkmodel=True, verbose=3):
+    """Import Directed Acyclic Graph.
+
+    Parameters
+    ----------
+    filepath : str, (default: sprinkler)
+        Pre-defined examples are depicted below, or provide the absolute file path to the .bif model file.. The default is 'sprinkler'.
+        'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'filepath/to/model.bif',
+    CPD : bool, optional
+        Directed Acyclic Graph (DAG). The default is True.
+    checkmodel : bool
+        Check the validity of the model. The default is True
+    verbose : int, optional
+        Print progress to screen. The default is 3.
+        0: None, 1: ERROR, 2: WARN, 3: INFO (default), 4: DEBUG, 5: TRACE
+
+    Returns
+    -------
+    dict containing model and adjmat.
+        model : BayesianModel
+        adjmat : Adjacency matrix
+
+    Examples
+    --------
+    >>> import bnlearn as bn
+    >>> model = bn.import_DAG('sprinkler')
+    >>> bn.plot(model)
+
+    """
+    PATH_TO_DATA = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
+    out={}
+    model=None
+    filepath=filepath.lower()
+    if verbose>=3: print('[bnlearn] >Import <%s>' %(filepath))
+
+    # Load data
+    if filepath=='sprinkler':
+        model = _DAG_sprinkler(CPD=CPD)
+    elif (filepath=='asia') or (filepath=='alarm') or (filepath=='andes') or (filepath=='sachs') or (filepath=='water'):
+        getfile = os.path.join(PATH_TO_DATA, filepath+'.bif')
+        if not os.path.isfile(getfile):
+            PATH_TO_DATA = _download_example(filepath, verbose=3)
+            getPath = _unzip(PATH_TO_DATA, verbose=verbose)
+        model = _bif2bayesian(getfile, verbose=verbose)
+    # elif filepath=='miserables':
+    #     getfile = os.path.join(PATH_TO_DATA, filepath+'.json')
+    #     if not os.path.isfile(getfile):
+    #         PATH_TO_DATA = _download_example(filepath, verbose=3)
+    #         getPath = _unzip(PATH_TO_DATA, ext='.json', verbose=verbose)
+
+    #     f = open(os.path.join(PATH_TO_DATA, 'miserables.json'))
+    #     data = json.loads(f.read())
+    #     L=len(data['links'])
+    #     edges=[(data['links'][k]['source'], data['links'][k]['target']) for k in range(L)]
+    #     model=nx.Graph(edges, directed=False)
+    else:
+        if os.path.isfile(filepath):
+            model = _bif2bayesian(filepath, verbose=verbose)
+        else:
+            if verbose>=3: print('[bnlearn] >filepath does not exist! <%s>' %(filepath))
+            return(out)
+
+    # Setup adjacency matrix
+    adjmat = _dag2adjmat(model)
+
+    # Store
+    out['model']=model
+    out['adjmat']=adjmat
+
+    # check_model check for the model structure and the associated CPD and returns True if everything is correct otherwise throws an exception
+    if (model is not None) and CPD and checkmodel:
+        _check_model(out['model'], verbose=verbose)
+        if verbose>=4:
+            print_CPD(out)
+
+    return(out)
+
+
+# %% unzip
+def _unzip(getZip, ext='.bif', verbose=3):
+    if not os.path.isdir(getZip):
+        if verbose>=3: print('[bnlearn] >Extracting files..')
+        [pathname, _] = os.path.split(getZip)
+        # Unzip
+        zip_ref = zipfile.ZipFile(getZip, 'r')
+        zip_ref.extractall(pathname)
+        zip_ref.close()
+        getPath = getZip.replace('.zip', ext)
+        if not os.path.isfile(getPath):
+            getPath = None
+
+    return getPath
 
 
 # %% Pre-processing of input raw dataset
@@ -714,10 +825,10 @@ def df2onehot(df, y_min=10, perc_min_num=0.8, dtypes='pandas', excl_background=N
         One-hot dataframe.
 
     """
-    from df2onehot import df2onehot
+    from df2onehot import df2onehot as df2hot
 
     # Convert dataframe to onehot by keeping only categorical variables.
-    out = df2onehot(df, y_min=y_min, perc_min_num=perc_min_num, dtypes=dtypes, excl_background=excl_background, hot_only=True, verbose=verbose)
+    out = df2hot(df, y_min=y_min, perc_min_num=perc_min_num, dtypes=dtypes, excl_background=excl_background, hot_only=True, verbose=verbose)
     # Numerical
     df_num = out['numeric'].iloc[:, out['dtypes']=='cat']
     df_num = df_num.astype(int)
@@ -739,3 +850,229 @@ def _filter_df(adjmat, df, verbose=3):
         if verbose>=3: print('[bnlearn] >Removing columns from dataframe to make consistent with DAG [%s]' %(remcols))
         df.drop(labels=remcols, axis=1, inplace=True)
     return df
+
+
+# %% Make prediction in inference model
+def predict(model, df, variables, to_df=True, method='max', verbose=3):
+    """Predict on data from a Bayesian network.
+    
+    Description
+    -----------
+    The inference on the dataset is performed sample-wise by using all the available nodes as evidence (obviously, with the exception of the node whose values we are predicting).
+    The states with highest probability are returned.
+
+    Parameters
+    ----------
+    model : Object
+        An object of class from bn.fit.
+    df : pd.DataFrame
+        Each row in the DataFrame will be predicted
+    variables : str or list of str
+        The label(s) of node(s) to be predicted.
+    to_df : Bool, (default is True)
+        The output is converted to dataframe output. Note that this heavily impacts the speed. 
+    method : str
+        The method that is used to select the for the inferences.
+        'max' : Return the variable values based on the maximum probability.
+        None : Returns all Probabilities
+    verbose : int, optional
+        Print progress to screen. The default is 3.
+        0: None, 1: ERROR, 2: WARN, 3: INFO (default), 4: DEBUG, 5: TRACE
+
+    Returns
+    -------
+    P : dict or DataFrame
+        Predict() returns a dict with the evidence and states that resulted in the highest probability for the input variable.
+    
+    Examples
+    --------
+    >>> import bnlearn as bn
+    >>> model = bn.import_DAG('sprinkler')
+    >>>
+    >>> # Make single inference
+    >>> query = bn.inference.fit(model, variables=['Rain', 'Cloudy'], evidence={'Wet_Grass':1})
+    >>> print(query)
+    >>> print(bn.query2df(query))
+    >>> 
+    >>> # Lets create an example dataset with 100 samples and make inferences on the entire dataset.
+    >>> df = bn.sampling(model, n=1000)
+    >>>
+    >>> # Each sample will be assesed and the states with highest probability are returned.
+    >>> Pout = bn.predict(model, df, variables=['Rain', 'Cloudy'])
+    >>>
+    >>> print(Pout)
+    >>> #     Cloudy  Rain         p
+    >>> # 0        0     0  0.647249
+    >>> # 1        0     0  0.604230
+    >>> # ..     ...   ...       ...
+    >>> # 998      0     0  0.604230
+    >>> # 999      1     1  0.878049
+
+    """
+    if not isinstance(df, pd.DataFrame): raise Exception('[bnlearn] >Error: Input requires a pd.DataFrame.')
+    if not isinstance(model, dict): raise Exception('[bnlearn] >Error: Input requires a dict that contains the key: model.')
+    if isinstance(variables, str): variables=[variables]
+    # Remove columns that are used as priors
+    dfX = df.loc[:, ~np.isin(df.columns.values, variables)]
+    if verbose>=3: print('[bnlearn]> Remaining columns for inference: %d' %(dfX.shape[1]))
+
+    # Get only the unique records in the DataFrame to reduce computation time.
+    dfU = dfX.drop_duplicates()
+    # Make empty array
+    P = np.array([None]*dfX.shape[0])
+    for i in tqdm(range(dfU.shape[0])):
+        # Get input data and create a dict.
+        evidence = dfU.iloc[i,:].to_dict()
+        # Do the inference.
+        query = bnlearn.inference.fit(model, variables=variables, evidence=evidence, to_df=False, verbose=0)
+        # Find original location of the input data.
+        loc = np.sum((dfX==dfU.iloc[i,:]).values, axis=1)==dfU.shape[1]
+        # Store inference
+        P[loc] = _get_prob(query, method=method)
+    P = list(P)
+
+    # Loop the dataframe
+    # P1 = []
+    # for i in tqdm(range(dfX.shape[0])):
+    #     # Setup input data
+    #     evidence = dfX.iloc[i,:].to_dict()
+    #     # Do the inferemce
+    #     query = inference.fit(model, variables=variables, evidence=evidence, to_df=False, verbose=0)
+    #     # Store in list
+    #     P1.append(_get_max_prob(query))
+
+    if to_df: P = pd.DataFrame(P)
+    return P
+
+
+# %% 
+def _get_prob(query, method='max'):
+    # Setup all combinations
+    allcomb = np.array(list(itertools.product([0, 1], repeat=len(query.variables))))
+    # Get highest P-value and gather data
+    Pq = query.values.flatten()
+    if method=='max':
+        idx = np.argmax(Pq)
+        comb = allcomb[idx]
+        p = Pq[idx]
+        # Store in dict
+        out = dict(zip(query.variables, comb))
+        out['p']=p
+    else:
+        out = bnlearn.query2df(query).to_dict()
+    return out
+
+
+# %%
+def query2df(query):
+    """Convert query from inference model to a dataframe.
+
+    Parameters
+    ----------
+    query : Object from the inference model.
+        Convert query object to a dataframe.
+
+    Returns
+    -------
+    df : pd.DataFrame()
+        Dataframe with inferences.
+
+    """
+    df = pd.DataFrame(data = list(itertools.product([0, 1], repeat=len(query.variables))), columns=query.variables)
+    df['p'] = query.values.flatten()
+    return df
+
+# %% Comparison of two networks
+# def _compare_networks(adjmat_true, adjmat_pred, pos=None, showfig=True, width=15, height=8, verbose=3):
+#     # Make sure columns and indices to match
+#     [IArow,IBrow]=ismember(adjmat_true.index.values, adjmat_pred.index.values)
+#     [IAcol,IBcol]=ismember(adjmat_true.columns.values, adjmat_pred.columns.values)
+#     adjmat_true = adjmat_true.loc[IArow,IAcol]
+#     adjmat_pred = adjmat_pred.iloc[IBrow,IBcol]
+    
+#     # Make sure it is boolean adjmat
+#     adjmat_true = adjmat_true>0
+#     adjmat_pred = adjmat_pred>0
+
+#     # Check whether order is correct
+#     assert np.all(adjmat_true.columns.values==adjmat_pred.columns.values), 'Column order of both input values could not be matched'
+#     assert np.all(adjmat_true.index.values==adjmat_pred.index.values), 'Row order of both input values could not be matched'
+    
+#     # Get edges
+#     y_true = adjmat_true.stack().reset_index()[0].values
+#     y_pred = adjmat_pred.stack().reset_index()[0].values
+    
+#     # Confusion matrix
+#     scores=confmatrix.twoclass(y_true, y_pred, threshold=0.5, classnames=['Disconnected','Connected'], title='', cmap=plt.cm.Blues, showfig=1, verbose=0)
+#     #bayes.plot(out_bayes['adjmat'], pos=G['pos'])
+    
+#     # Setup graph
+#     adjmat_diff = adjmat_true.astype(int)
+#     adjmat_diff[(adjmat_true.astype(int) - adjmat_pred.astype(int))<0]=2
+#     adjmat_diff[(adjmat_true.astype(int) - adjmat_pred.astype(int))>0]=-1
+    
+#     if showfig:
+#         # Setup graph
+#     #    G_true = adjmat2graph(adjmat_true)
+#         G_diff = adjmat2graph(adjmat_diff)
+#         # Graph layout
+#         pos = graphlayout(G_diff, pos=pos, scale=1, layout='fruchterman_reingold', verbose=verbose)
+#         # Bootup figure
+#         plt.figure(figsize=(width,height))
+#         # nodes
+#         nx.draw_networkx_nodes(G_diff, pos, node_size=700)
+#         # edges
+#         colors  = [G_diff[u][v]['color'] for u,v in G_diff.edges()]
+#         #weights = [G_diff[u][v]['weight'] for u,v in G_diff.edges()]
+#         nx.draw_networkx_edges(G_diff, pos, arrowstyle='->', edge_color=colors, width=1)
+#         # Labels
+#         nx.draw_networkx_labels(G_diff, pos, font_size=20, font_family='sans-serif')
+#         # Get labels of weights
+#         #labels = nx.get_edge_attributes(G,'weight')
+#         # Plot weights
+#         nx.draw_networkx_edge_labels(G_diff, pos, edge_labels=nx.get_edge_attributes(G_diff,'weight'))
+#         # Making figure nice
+#         #plt.legend(['Nodes','TN','FP','test'])
+#         ax = plt.gca()
+#         ax.set_axis_off()
+#         plt.show()
+    
+#     # Return
+#     return(scores, adjmat_diff)
+
+# # Make graph layout
+# def graphlayout(model, pos, scale=1, layout='fruchterman_reingold', verbose=3):
+#     if isinstance(pos, type(None)):
+#         if layout=='fruchterman_reingold':
+#             pos = nx.fruchterman_reingold_layout(model, scale=scale, iterations=50)
+#         else:
+#             pos = nx.spring_layout(model, scale=scale, iterations=50)
+#     else:
+#         if verbose>=3: print('[bnlearn] >Existing coordinates from <pos> are used.')
+
+#     return(pos)
+
+# def adjmat2graph(adjmat):
+#     G = nx.DiGraph() # Directed graph
+#     # Convert adjmat to source target
+#     df_edges=adjmat.stack().reset_index()
+#     df_edges.columns=['source', 'target', 'weight']
+#     df_edges['weight']=df_edges['weight'].astype(float)
+    
+#     # Add directed edge with weigth
+#     for i in range(df_edges.shape[0]):
+#         if df_edges['weight'].iloc[i]!=0:
+#             # Setup color
+#             if df_edges['weight'].iloc[i]==1:
+#                 color='k'
+#             elif df_edges['weight'].iloc[i]>1:
+#                 color='r'
+#             elif df_edges['weight'].iloc[i]<0:
+#                 color='b'
+#             else:
+#                 color='p'
+            
+#             # Create edge in graph
+#             G.add_edge(df_edges['source'].iloc[i], df_edges['target'].iloc[i], weight=np.abs(df_edges['weight'].iloc[i]), color=color)    
+#     # Return
+#     return(G)
