@@ -1,16 +1,81 @@
 # %%
 import bnlearn as bn
-print(bn.__version__)
-# print(dir(bn))
+# print(bn.__version__)
+print(dir(bn))
 
-# # %%
 # print(dir(bn.structure_learning))
 # print(dir(bn.parameter_learning))
 # print(dir(bn.inference))
 
+
+# %%
+import bnlearn as bn
+examples = ['titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'water', 'miserables']
+for example in examples:
+    df = bn.import_DAG(example)
+    # assert ~df.empty
+
+# %% Download example
+import bnlearn as bn
+examples = ['titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'water', 'miserables']
+for example in examples:
+    df = bn.import_example(data=example)
+    # assert ~df.empty
+
 # %%
 import bnlearn as bn
 df = bn.import_example()
+model = bn.structure_learning.fit(df)
+model = bn.structure_learning.fit(df, methodtype='hc')
+ 
+# %% Predict
+import bnlearn as bn
+
+df = bn.import_example('asia')
+edges = [('smoke', 'lung'),
+         ('smoke', 'bronc'),
+         ('lung', 'xray'),
+         ('bronc', 'xray')]
+
+# Make the actual Bayesian DAG
+DAG = bn.make_DAG(edges, verbose=0)
+model = bn.parameter_learning.fit(DAG, df, verbose=3)
+# Generate some data based on DAG
+df = bn.sampling(model, n=1000)
+# Make predictions
+Pout = bn.predict(model, df, variables=['bronc','xray'])
+# query = bnlearn.inference.fit(model, variables=['bronc','xray'], evidence=evidence, to_df=False, verbose=0)
+# print(query)
+
+
+# %% topological sort example
+edges = [('1', '2'),
+         ('1', '3'),
+         ('2', '4'),
+         ('2', '3'),
+         ('3', '4'),
+         ('3', '5'),
+         ]
+
+# Make the actual Bayesian DAG
+DAG = bn.make_DAG(edges, verbose=0)
+# Plot
+bn.plot(DAG)
+# Topological ordering
+bn.topological_sort(DAG)
+
+bn.topological_sort(DAG, '3')
+
+# %%
+import bnlearn as bn
+DAG = bn.import_DAG('sprinkler', verbose=0)
+
+bn.topological_sort(DAG, 'Rain')
+bn.topological_sort(DAG)
+
+# Different inputs
+bn.topological_sort(DAG['adjmat'], 'Rain')
+bn.topological_sort(bn.adjmat2vec(DAG['adjmat']), 'Rain')
 
 # %%
 
@@ -20,6 +85,8 @@ df = bn.sampling(DAG, n=1000, verbose=0)
 model = bn.structure_learning.fit(df, methodtype='chow-liu', root_node='Wet_Grass')
 G = bn.plot(model)
 
+bn.topological_sort(model, 'Rain')
+
 # %%
 # Example dataframe sprinkler_data.csv can be loaded with:
 df = bn.import_example()
@@ -62,6 +129,9 @@ G = bn.plot(model, verbose=0)
 model_hc_bic  = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic', verbose=0)
 
 # %% Try all methods vs score types
+import bnlearn as bn
+df = bn.import_example()
+
 model_hc_bic = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic')
 model_hc_k2 = bn.structure_learning.fit(df, methodtype='hc', scoretype='k2')
 model_hc_bdeu = bn.structure_learning.fit(df, methodtype='hc', scoretype='bdeu')
@@ -72,7 +142,9 @@ model_cs_k2 = bn.structure_learning.fit(df, methodtype='cs', scoretype='k2')
 model_cs_bdeu = bn.structure_learning.fit(df, methodtype='cs', scoretype='bdeu')
 model_cl = bn.structure_learning.fit(df, methodtype='cl', root_node='Cloudy')
 
-bn.compare_networks(model, model_hc_bic, pos=G['pos'], verbose=0)
+G = bn.plot(model_hc_bic, verbose=0)
+
+bn.compare_networks(model_hc_bic, model_cl, pos=G['pos'], verbose=0)
 
 # %% Example with dataset
 import bnlearn as bn
@@ -107,7 +179,7 @@ bn.plot(DAG, verbose=0)
 bn.print_CPD(DAG)
 
 # Sampling
-df_sampling = bn.sampling(DAG, n=1000)
+# df_sampling = bn.sampling(DAG, n=1000)
 
 # Learn its parameters from data and perform the inference.
 DAG = bn.parameter_learning.fit(DAG, df, verbose=3)
@@ -124,6 +196,10 @@ q3 = bn.inference.fit(DAG, variables=['lung'], evidence={'smoke':1, 'bronc':1})
 q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xray':0})
 q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':0, 'xray':0})
 
+bn.topological_sort(DAG)
+
+bn.query2df(q4)
+
 # DAGmle = bn.parameter_learning.fit(DAG, df, methodtype='maximumlikelihood')
 # bn.print_CPD(DAGmle)
 # bn.print_CPD(DAGbay)
@@ -137,6 +213,16 @@ q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':0, 'xra
 # bn.compare_networks(DAGbay, DAGnew)
 # bn.compare_networks(DAGnew, DAGbay)
 
+# %% predict
+
+df = bn.sampling(DAG, n=100)
+out = bn.predict(DAG, df, variables='bronc')
+out = bn.predict(DAG, df, variables=['bronc','xray'])
+out = bn.predict(DAG, df, variables=['bronc','xray','smoke'])
+
+print('done\n\n')
+print(out)
+
 # %% compute causalities
 # Load asia DAG
 import bnlearn as bn
@@ -151,18 +237,21 @@ bn.plot(model, verbose=0)
 # Print the CPDs
 bn.print_CPD(model)
 # Comparison
-bn.compare_networks(DAG, model, verbose=0)
-bn.compare_networks(model, DAG)
 
 # Learn its parameters from data and perform the inference.
-DAGnew = bn.parameter_learning.fit(model, df, methodtype='bayes', verbose=0)
+DAG = bn.parameter_learning.fit(model, df, methodtype='bayes', verbose=0)
 # Print the CPDs
-bn.print_CPD(DAGnew)
+bn.print_CPD(DAG)
+
+# Nothing is changed for the DAG. Only the CPDs are estimated now.
+bn.compare_networks(DAG, model, verbose=0)
 
 # Make inference
-q4 = bn.inference.fit(DAGnew, variables=['bronc','lung'], evidence={'smoke':1, 'xray':0}, verbose=0)
+q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xray':0}, verbose=3)
+# q4 = bn.inference.fit(DAG, variables=['bronc','lung','xray'], evidence={'smoke':1}, verbose=3)
 # q4 = bn.inference.fit(DAGnew, variables=['bronc','lung'], evidence={'smoke':0, 'xray':0})
 
+# pd.DataFrame(index=q4.variables, data=q4.values, columns=q4.variables)
 
 # %% Example compare networks
 # Load asia DAG
@@ -181,7 +270,7 @@ bn.compare_networks(model, model_sl, pos=G['pos'])
 
 
 # Structure learning with black list
-model_wl = bn.structure_learning.fit(df, methodtype='ex', white_list=['asia','tub','bronc','xray','smoke'], bw_list_method='enforce')
+model_wl = bn.structure_learning.fit(df, methodtype='hc', white_list=['asia','tub','bronc','xray','smoke'], bw_list_method='enforce')
 bn.plot(model_wl, pos=G['pos'])
 
 model_bl = bn.structure_learning.fit(df, methodtype='hc', black_list=['asia','tub'], bw_list_method='enforce')
@@ -238,7 +327,9 @@ q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain':1, 'Sprinkl
 q2 = bn.inference.fit(DAG, variables=['Wet_Grass','Rain'], evidence={'Sprinkler': 1})
 
 print(q1)
+print(q1.df)
 print(q2)
+print(q2.df)
 
 
 # %% INFERENCE 2
@@ -250,7 +341,9 @@ q2 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xra
 q3 = bn.inference.fit(DAG, variables=['lung'], evidence={'bronc':1, 'smoke':1})
 
 print(q1)
+print(q1.df)
 print(q2)
+print(q2.df)
 
 
 # %% Example with mixed dataset: titanic case
@@ -260,22 +353,29 @@ df_raw = bn.import_example(data='titanic')
 # Convert to onehot
 dfhot, dfnum = bn.df2onehot(df_raw)
 # Structure learning
-DAG = bn.structure_learning.fit(dfnum, methodtype='bayes', black_list=['Pclass','Sex','Embarked','Parch','Name'], root_node='Survived')
+# DAG = bn.structure_learning.fit(dfnum, methodtype='cl', black_list=['Embarked','Parch','Name'], root_node='Survived', bw_list_method='filter')
+DAG = bn.structure_learning.fit(dfnum, methodtype='hc', black_list=['Embarked','Parch','Name'], bw_list_method='filter')
 # Plot
 G = bn.plot(DAG)
 # Parameter learning
 model = bn.parameter_learning.fit(DAG, dfnum)
 # Make inference
-q1 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex':1, 'Pclass':1})
-q1 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex':0})
+q1 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex':True, 'Pclass':True}, verbose=0)
+q2 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex':0}, verbose=0)
 
-bn.print_CPD(model)
+print(q1)
+print(q1.df)
+# bn.print_CPD(model)
+
+# Create test dataset
+Xtest = bn.sampling(model, n=100)
+# Predict the whole dataset
+Pout = bn.predict(model, Xtest, variables=['Survived'])
 
-df = bn.sampling(DAG, n=1000)
 
 # %%
 import bnlearn as bn
-DAG = bn.import_DAG('sprinkler', CPD=False)
+DAG = bn.import_DAG('sprinkler', CPD=True)
 # DAG = bn.import_DAG('asia')
 bn.plot(DAG)
 bn.print_CPD(DAG)
