@@ -23,7 +23,8 @@ else:
     from pgmpy.estimators import ConstraintBasedEstimator
 
 # from packaging import version
-from bnlearn.bnlearn import _dag2adjmat
+# from bnlearn.bnlearn import _dag2adjmat
+import bnlearn
 
 
 # %% Structure Learning
@@ -71,7 +72,7 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
     white_list : List or None, (default : None)
         List of edges are white listed by either by filtering or enforcing (see bw_list_method).
         In case of filtering, the search is limited to those edges. The resulting model will then only contain edges that are in white_list.
-        Works only in case of methodtype='hc'/ See also paramter: `bw_list_method`
+        Works only in case of methodtype='hc' See also paramter: `bw_list_method`
     bw_list_method : str, (default : None)
         'enforce' : A list of edges can optionally be passed as `black_list` or `white_list` to exclude those edges or to limit the search. This option is limited to only methodtype='hc'
         'filter' : Filter the dataframe based on `black_list` or `white_list`. Filtering can be done for every methodtype/scoretype.
@@ -198,7 +199,7 @@ def fit(df, methodtype='hc', scoretype='bic', black_list=None, white_list=None,
         out = _chowliu(df, config['root_node'], verbose=config['verbose'])
 
     # Setup simmilarity matrix
-    adjmat = _dag2adjmat(out['model'])
+    adjmat = bnlearn._dag2adjmat(out['model'])
 
     # adjmat = pd.DataFrame(data=False, index=out['model'].nodes(), columns=out['model'].nodes()).astype('bool')
     # # Fill adjmat with edges
@@ -351,24 +352,17 @@ def _hillclimbsearch(df, scoretype='bic', black_list=None, white_list=None, max_
     # Set scoring type
     scoring_method = _SetScoringType(df, scoretype, verbose=verbose)
     # Set search algorithm
-    model = HillClimbSearch(df, scoring_method=scoring_method)
-
-    # Compute best DAG
-    # PGMPY_VER = version.parse(pgmpy.__version__)>version.parse("0.1.9")
-    # if PGMPY_VER:
-    #     best_model = model.estimate(max_indegree=max_indegree, black_list=black_list, white_list=white_list)
-    # else:
-    #     best_model = model.estimate(max_indegree=max_indegree)  # Can be be removed if pgmpy >v0.1.9
+    model = HillClimbSearch(df)
 
     # Compute best DAG
     if bw_list_method=='enforce':
         if (black_list is not None) or (white_list is not None):
             if verbose>=3: print('[bnlearn] >Enforcing nodes based on black_list and/or white_list.')
         # best_model = model.estimate()
-        best_model = model.estimate(max_indegree=max_indegree, tabu_length=tabu_length, epsilon=epsilon, max_iter=max_iter, black_list=black_list, white_list=white_list, show_progress=False)
+        best_model = model.estimate(scoring_method=scoring_method, max_indegree=max_indegree, tabu_length=tabu_length, epsilon=epsilon, max_iter=max_iter, black_list=black_list, white_list=white_list, show_progress=False)
     else:
         # At this point, variables are readily filtered based on bw_list_method or not (if nothing defined).
-        best_model = model.estimate(max_indegree=max_indegree, tabu_length=tabu_length, epsilon=epsilon, max_iter=max_iter, show_progress=False)
+        best_model = model.estimate(scoring_method=scoring_method, max_indegree=max_indegree, tabu_length=tabu_length, epsilon=epsilon, max_iter=max_iter, show_progress=False)
 
     # Store
     out['model']=best_model
